@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 1998-2020 Sergey Poznyakoff
+   Copyright (C) 1998-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -631,7 +631,7 @@ set_database(enum grecs_callback_command cmd,
 	    dicod_database_free(dict);
 	    return 1;
 	}
-	
+
 	if (!database_list) {
 	    database_list = xdico_list_create();
 	    dico_list_set_comparator (database_list, cmp_database_name, NULL);
@@ -878,6 +878,12 @@ struct grecs_keyword kwd_database[] = {
       NULL, offsetof(dicod_database_t, extra),
       member_database_cb },
 
+    { "conv", N_("string"),
+      N_("List of input converters"),
+      grecs_type_string, GRECS_LIST,
+      NULL, offsetof(dicod_database_t, conv),
+      cb_dico_list },
+
     { NULL }
 };
 
@@ -1105,16 +1111,16 @@ strategy_deny_all_cb(enum grecs_callback_command cmd,
 		     grecs_value_t *value,
 		     void *cb_data)
 {
-    int bool;
+    int bv;
 
     if (cmd != grecs_callback_set_value) {
 	grecs_error(locus, 0, _("Unexpected block statement"));
 	return 1;
     }
     if (value->type == GRECS_TYPE_STRING
-	&& grecs_string_convert(&bool, grecs_type_bool,
+	&& grecs_string_convert(&bv, grecs_type_bool,
 				value->v.string, locus) == 0) {
-	if (bool)
+	if (bv)
 	    stratcl_add_disable(*(dico_list_t*) varptr);
     } else
 	grecs_error(locus, 0, _("Expected boolean value"));
@@ -1760,8 +1766,7 @@ main(int argc, char **argv)
 	dicod_log_setup();
 
     if (mode == MODE_PREPROC)
-	exit(grecs_preproc_run(config_file, grecs_preprocessor) ?
-	      EX_CONFIG : 0);
+	exit(grecs_preprocess(config_file, 0) ? EX_CONFIG : 0);
     else if (mode == MODE_TEST) {
 	dicod_loader_init();
 	exit(dicod_module_test(argc, argv));
