@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008-2021 Sergey Poznyakoff
+   Copyright (C) 2008-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ off_t total_bytes_out;
 
 #define OSTREAM_INITIALIZED       0x01
 #define OSTREAM_DESTROY_TRANSPORT 0x02
+#define OSTREAM_NEWLINE           0x04
 
 struct ostream {
     dico_stream_t transport;
@@ -28,16 +29,34 @@ struct ostream {
     dico_assoc_list_t headers;
 };
 
+static dico_stream_t
+content_type_transport(char const *content_type, dico_stream_t transport)
+{
+    size_t len;
+
+    len = strcspn(content_type, "/");
+    if (len == 4 && strncasecmp(content_type, "text", len) == 0) {
+	if (content_type[len] == 0)
+	    return dico_linetrim_stream(transport, 1024, 1);
+	else {
+	    content_type += len + 1;
+	    len = strcspn(content_type, ";");
+	    if (len == 5 && strncasecmp(content_type, "plain", len) == 0)
+		return dico_linetrim_stream(transport, 1024, 1);
+	}
+    }
+    return NULL;
+}
+
 static int
 print_headers(struct ostream *ostr)
 {
     int rc = 0;
-    const char *enc;
-    
+
     if (ostr->headers) {
 	dico_iterator_t itr;
 	struct dico_assoc *p;
-	
+
 	itr = dico_assoc_iterator(ostr->headers);
 	for (p = dico_iterator_first(itr); p; p = dico_iterator_next(itr)) {
 	    dico_stream_write(ostr->transport, p->key, strlen(p->key));
@@ -47,41 +66,49 @@ print_headers(struct ostream *ostr)
 	}
 	dico_iterator_destroy(&itr);
     }
-    
+
     rc = dico_stream_write(ostr->transport, "\n", 1);
-    
+
     if (rc == 0) {
+	const char *enc;
 	dico_stream_t str;
-	
+
 	if ((enc = dico_assoc_find(ostr->headers,
 				   CONTENT_TRANSFER_ENCODING_HEADER))
 	    && strcmp(enc, "8bit")) {
 	    str = dico_codec_stream_create(enc, FILTER_ENCODE,
 					   ostr->transport);
-	} else {
+	} else if ((enc = dico_assoc_find(ostr->headers,
+					  CONTENT_TYPE_HEADER)) != NULL)
+	    str = content_type_transport(enc, ostr->transport);
+	else
 	    str = dico_linetrim_stream(ostr->transport, 1024, 1);
-	}
+
 	if (str) {
 	    ostr->transport = str;
 	    ostr->flags |= OSTREAM_DESTROY_TRANSPORT;
-	}    
+	}
     }
     return rc;
 }
 
-
 static int
 ostream_write(void *data, const char *buf, size_t size, size_t *pret)
 {
     struct ostream *ostr = data;
-    
+
     if (!(ostr->flags & OSTREAM_INITIALIZED)) {
 	if (option_mime && print_headers(ostr))
 	    return dico_stream_last_error(ostr->transport);
 	ostr->flags |= OSTREAM_INITIALIZED;
     }
-    if (buf[0] == '.' && dico_stream_write(ostr->transport, ".", 1))
-	return dico_stream_last_error(ostr->transport);
+    if (ostr->flags & OSTREAM_NEWLINE) {
+	ostr->flags &= ~OSTREAM_NEWLINE;
+	if (buf[0] == '.' && dico_stream_write(ostr->transport, ".", 1))
+	    return dico_stream_last_error(ostr->transport);
+    }
+    if (buf[size-1] == '\n')
+	ostr->flags |= OSTREAM_NEWLINE;
     *pret = size;
     return dico_stream_write(ostr->transport, buf, size);
 }
@@ -119,9 +146,9 @@ ostream_ioctl(void *data, int code, void *call_data)
     case DICO_IOCTL_BYTES_OUT:
 	dico_stream_flush(ostr->transport);
 	*(off_t*)call_data = dico_stream_bytes_out(ostr->transport) -
-	                       ostr->nout;
+			       ostr->nout;
 	break;
-	
+
     default:
 	errno = EINVAL;
 	return -1;
@@ -134,7 +161,7 @@ dicod_ostream_create(dico_stream_t str, dico_assoc_list_t headers)
 {
     struct ostream *ostr = xmalloc(sizeof(*ostr));
     dico_stream_t stream;
-    
+
     if (dico_stream_create(&stream, DICO_STREAM_WRITE, ostr))
 	xalloc_die();
     ostr->transport = str;
@@ -148,5 +175,3 @@ dicod_ostream_create(dico_stream_t str, dico_assoc_list_t headers)
     dico_stream_set_buffer(stream, dico_buffer_line, 1024);
     return stream;
 }
-
-    
