@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2012-2018 Sergey Poznyakoff
+   Copyright (C) 2012-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,18 +14,18 @@
    You should have received a copy of the GNU General Public License
    along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <dico.h>
 #include "gcide.h"
 #include <errno.h>
 #include <appi18n.h>
+#include <assert.h>
 
 struct gcide_idx_cache {
     size_t pageno;
@@ -46,15 +46,23 @@ struct gcide_idx_file {
 #define REF_NOT_FOUND ((size_t)-1)
 
 static void
-_free_index(struct gcide_idx_file *file)
+_free_cache(struct gcide_idx_file *file)
 {
     size_t i;
-    free(file->name);
     for (i = 0; i < file->cache_used; i++) {
 	free(file->cache[i]->page);
 	free(file->cache[i]);
     }
     free(file->cache);
+    file->cache = NULL;
+    file->cache_used = 0;
+}
+
+static void
+_free_index(struct gcide_idx_file *file)
+{
+    _free_cache(file);
+    free(file->name);
     free(file);
 }
 
@@ -70,8 +78,9 @@ _idx_full_read(struct gcide_idx_file *file, void *buf, size_t size)
 	    dico_log(L_ERR, errno, _("error reading from `%s'"), file->name);
 	    return -1;
 	} else if (rc == 0) {
-	    dico_log(L_ERR, errno, _("short read while reading from `%s'"),
+	    dico_log(L_ERR, 0, _("short read while reading from `%s'"),
 		     file->name);
+	    errno = 0;
 	    return -1;
 	}
 	p += rc;
@@ -80,63 +89,75 @@ _idx_full_read(struct gcide_idx_file *file, void *buf, size_t size)
     return 0;
 }
 
-static int
-_open_index(struct gcide_idx_file *file)
+int
+gcide_idx_fileno(struct gcide_idx_file *file)
+{
+    return file->fd;
+}
+
+int
+gcide_idx_reopen(struct gcide_idx_file *file)
 {
     off_t total;
     struct stat st;
-    
+
+    _free_cache(file);
+
+    if (lseek(file->fd, 0, SEEK_SET)) {
+	dico_log(L_ERR, errno, _("error rewinding `%s'"), file->name);
+	return IDXE_SYSERR;
+    }
+
     if (_idx_full_read(file, &file->header, sizeof(file->header)))
-	return 1;
+	return errno ? IDXE_SYSERR : IDXE_CORRUPT;
     if (memcmp(file->header.ihdr_magic, GCIDE_IDX_MAGIC,
-	       GCIDE_IDX_MAGIC_LEN)) {
-	dico_log(L_ERR, 0, _("file `%s' is not a valid gcide index file"),
-		 file->name);
-	return 1;
-    }
-    
-    if (fstat(file->fd, &st)) {
-	dico_log(L_ERR, errno, "fstat `%s'", file->name);
-	return 1;
-    }
-    
+	       GCIDE_IDX_MAGIC_LEN))
+	return IDXE_BADFILE;
+
+    if (file->header.ihdr_version != GCIDE_IDX_VERSION)
+	return IDXE_BADVER;
+
+    if (fstat(file->fd, &st))
+	return IDXE_SYSERR;
+
     total = (file->header.ihdr_num_pages + 1) * file->header.ihdr_pagesize;
-    if (total != st.st_size) {
-	dico_log(L_ERR, 0, _("index file `%s' is corrupted"), file->name);
-	return 1;
-    }
-    return 0;
+    if (total != st.st_size)
+	return IDXE_CORRUPT;
+
+    return IDXE_OK;
 }
 
-struct gcide_idx_file *
-gcide_idx_file_open(const char *name, size_t cachesize)
+int
+gcide_idx_file_open(const char *name, size_t cachesize, int rw,
+		    struct gcide_idx_file **ret_file)
 {
-    int fd;
+    int rc, ec;
     struct gcide_idx_file *file;
-    
+
     file = calloc(1, sizeof(*file));
-    if (!file) {
-        DICO_LOG_ERRNO();
-	return NULL;
-    }
+    if (!file)
+	return IDXE_SYSERR;
+
+    rc = IDXE_SYSERR;
     file->name = strdup(name);
-    if (!file->name) {
-        DICO_LOG_ERRNO();
-	free(file);
-	return NULL;
+    if (file->name) {
+	int fd = open(name, rw ? O_RDWR : O_RDONLY);
+	if (fd != -1) {
+	    file->fd = fd;
+	    rc = gcide_idx_reopen(file);
+	    if (rc == IDXE_OK) {
+		file->cache_size = cachesize;
+		*ret_file = file;
+		return rc;
+	    }
+	}
     }
-    fd = open(name, O_RDONLY);
-    if (fd == -1) {
-	dico_log(L_ERR, errno, _("cannot open index file `%s'"), name);
-	free(file);
-    }
-    file->fd = fd;
-    if (_open_index(file)) {
-	_free_index(file);
-	return NULL;
-    }
-    file->cache_size = cachesize;
-    return file;
+
+    /* Error exit */
+    ec = errno;
+    _free_index(file);
+    errno = ec;
+    return rc;
 }
 
 void
@@ -148,6 +169,26 @@ gcide_idx_file_close(struct gcide_idx_file *file)
     }
 }
 
+int
+gcide_idx_lock(struct gcide_idx_file *file, int ex)
+{
+    struct flock fl;
+    fl.l_type = ex ? F_WRLCK : F_RDLCK;
+    fl.l_whence = SEEK_SET;
+    fl.l_start = fl.l_len = (off_t)0;
+    return fcntl (file->fd, F_SETLKW, &fl);
+}
+
+int
+gcide_idx_unlock(struct gcide_idx_file *file)
+{
+    struct flock fl;
+    fl.l_type = F_UNLCK;
+    fl.l_whence = SEEK_SET;
+    fl.l_start = fl.l_len = (off_t)0;
+    return fcntl (file->fd, F_SETLK, &fl);
+}
+
 size_t
 gcide_idx_headwords(struct gcide_idx_file *file)
 {
@@ -164,11 +205,11 @@ static struct gcide_idx_cache *
 _cache_alloc(struct gcide_idx_file *file)
 {
     struct gcide_idx_cache *cp;
-    
+
     if (!file->cache) {
 	file->cache = calloc(file->cache_size, sizeof(file->cache[0]));
 	if (!file->cache) {
-            DICO_LOG_ERRNO();
+	    DICO_LOG_ERRNO();
 	    return NULL;
 	}
     }
@@ -178,12 +219,12 @@ _cache_alloc(struct gcide_idx_file *file)
 	    return file->cache[file->cache_used - 1];
 	cp = calloc(1, sizeof(*cp));
 	if (!cp) {
-            DICO_LOG_ERRNO();
+	    DICO_LOG_ERRNO();
 	    return NULL;
 	}
 	cp->page = malloc(file->header.ihdr_pagesize);
 	if (!cp->page) {
-            DICO_LOG_ERRNO();
+	    DICO_LOG_ERRNO();
 	    free(cp);
 	    return NULL;
 	}
@@ -218,7 +259,7 @@ static struct gcide_idx_cache *
 _cache_find_page(struct gcide_idx_file *file, size_t n)
 {
     size_t i;
-    
+
     for (i = 0; i < file->cache_used; i++) {
 	if (file->cache[i]->pageno == n) {
 	    struct gcide_idx_cache *cp = file->cache[i];
@@ -226,7 +267,7 @@ _cache_find_page(struct gcide_idx_file *file, size_t n)
 	    return cp;
 	}
     }
-    
+
     return NULL;
 }
 
@@ -237,11 +278,11 @@ _idx_get_page(struct gcide_idx_file *file, size_t n)
     struct gcide_idx_cache *cp;
     struct gcide_idx_page *page;
     size_t i;
-    
+
     cp = _cache_find_page(file, n);
     if (cp)
 	return cp->page;
-    
+
     off = (n + 1) * file->header.ihdr_pagesize;
     if (lseek(file->fd, off, SEEK_SET) != off) {
 	dico_log(L_ERR, errno,
@@ -302,7 +343,7 @@ _idx_ref_locate(struct gcide_idx_file *file,
 {
     size_t l, u, idx;
     int res;
-    
+
     l = 0;
     u = page->ipg_header.hdr.phdr_numentries;
     while (l < u) {
@@ -323,12 +364,12 @@ _idx_page_locate(struct gcide_idx_file *file, char *headword, size_t hwlen)
 {
     size_t l, u, idx;
     int res;
-    
+
     l = 0;
     u = file->header.ihdr_num_pages;
     while (l < u) {
 	struct gcide_idx_page *page;
-	    
+
 	idx = (l + u) / 2;
 	page = _idx_get_page(file, idx);
 	if (!page)
@@ -367,12 +408,12 @@ struct gcide_iterator {
 				    if not yet determined. */
     size_t curref;               /* Position in iterator */
 
-    char *prevbuf;               /* Previous match */ 
+    char *prevbuf;               /* Previous match */
     size_t prevsize;
-    
+
     int flags;                   /* User-defined flags. */
 };
-    
+
 gcide_iterator_t
 gcide_idx_locate(struct gcide_idx_file *file, char *headword, size_t hwlen)
 {
@@ -410,10 +451,10 @@ gcide_idx_locate(struct gcide_idx_file *file, char *headword, size_t hwlen)
 	pageno++;
 	refno = 0;
     }
-    
+
     itr = malloc(sizeof(*itr));
     if (!itr) {
-        DICO_LOG_ERRNO();
+	DICO_LOG_ERRNO();
 	return NULL;
     }
     if (hwlen) {
@@ -424,20 +465,20 @@ gcide_idx_locate(struct gcide_idx_file *file, char *headword, size_t hwlen)
 	itr->headword = strdup(headword);
 
     if (!itr->headword) {
-        DICO_LOG_ERRNO();
+	DICO_LOG_ERRNO();
 	free(itr);
 	return NULL;
     }
 
     itr->hwlen = hwlen;
-    
+
     itr->file = file;
-    itr->start_pageno = itr->cur_pageno = pageno;    
+    itr->start_pageno = itr->cur_pageno = pageno;
     itr->start_refno = itr->cur_refno = refno;
     itr->page_numrefs = page->ipg_header.hdr.phdr_numentries;
     itr->curref = itr->numrefs = 0;
     itr->compare_count = file->compare_count;
-    
+
     return itr;
 }
 
@@ -498,7 +539,7 @@ gcide_iterator_rewind(gcide_iterator_t itr)
 
     if (!itr)
 	return -1;
-    itr->cur_pageno = itr->start_pageno;    
+    itr->cur_pageno = itr->start_pageno;
     itr->cur_refno = itr->start_refno;
     itr->curref = 0;
     page = _idx_get_page(itr->file, itr->cur_pageno);
@@ -512,7 +553,7 @@ struct gcide_ref *
 gcide_iterator_ref(gcide_iterator_t itr)
 {
     struct gcide_idx_page *page;
-    
+
     if (!itr)
 	return NULL;
     page = _idx_get_page(itr->file, itr->cur_pageno);
