@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2012-2018 Sergey Poznyakoff
+   Copyright (C) 2012-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,9 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <dico.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,7 +34,7 @@ struct nprefix {
     size_t skip;            /* Skip this number of matches */
     size_t count;           /* Display at most this number of matches */
     size_t n;               /* Number of the current match */
-    char *last_match_str;   /* Last matched string */    
+    char *last_match_str;   /* Last matched string */
     size_t last_match_len;  /* Size of last_match_str */
 };
 
@@ -46,15 +44,15 @@ nprefix_sel(int cmd, dico_key_t key, const char *dict_word)
     char const *key_word = key->word;
     struct nprefix *np;
     size_t wordlen;
-    
+
     switch (cmd) {
     case DICO_SELECT_BEGIN: {
 	char *p;
 	size_t skip, count;
-	
+
 	np = calloc(1, sizeof(np[0]));
 	if (!np) {
-            DICO_LOG_MEMERR();
+	    DICO_LOG_MEMERR();
 	    return 1;
 	}
 	np->prefix = (char*)key_word;
@@ -73,7 +71,7 @@ nprefix_sel(int cmd, dico_key_t key, const char *dict_word)
 	key->call_data = np;
 	break;
     }
-	
+
     case DICO_SELECT_RUN:
 	np = key->call_data;
 	if (np->last_match_str &&
@@ -89,7 +87,7 @@ nprefix_sel(int cmd, dico_key_t key, const char *dict_word)
 		char *p = realloc(np->last_match_str, s);
 		if (!p)
 		    return 0;
-		
+
 		np->last_match_str = p;
 		np->last_match_len = s;
 	    }
@@ -127,5 +125,3 @@ struct dico_database_module DICO_EXPORT(nprefix, module) = {
     DICO_CAPA_NODB,
     nprefix_init,
 };
-
-	
