@@ -1,7 +1,7 @@
 /* This file is part of GNU Dico.
    Copyright (C) 2008, 2012 Wojciech Polak
-   Copyright (C) 2019-2021 Sergey Poznyakoff
- 
+   Copyright (C) 2019-2024 Sergey Poznyakoff
+
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3, or (at your option)
@@ -55,23 +55,29 @@ find_method (PyMethodDef *methods, PyObject *self, const char *name)
 typedef struct {
     PyObject_HEAD;
     struct dico_key *key;
+    int key_alloc;
 } PySelectionKey;
 
-static void 
+static void
 _PySelectionKey_dealloc (PyObject *self)
 {
+    PySelectionKey *py_key = (PySelectionKey *)self;
+    if (py_key->key_alloc) {
+	dico_key_deinit(py_key->key);
+	free(py_key->key);
+    }
 }
 
 static PyMethodDef selection_key_methods[] = {
     /* None so far */
     { NULL, NULL, 0, NULL }
 };
-    
+
 static PyObject *
 _PySelectionKey_getattr (PyObject *self, char *name)
 {
     PySelectionKey *py_key = (PySelectionKey *)self;
-    
+
     if (strcmp (name, "word") == 0)
 	return PyUnicode_FromString (py_key->key->word);
     return find_method (selection_key_methods, self, name);
@@ -135,7 +141,7 @@ static PyMethodDef strategy_methods[] = {
     { NULL, NULL, 0, NULL }
 };
 
-static void 
+static void
 _PyStrategy_dealloc (PyObject *self)
 {
 }
@@ -183,7 +189,7 @@ static PyTypeObject PyStrategyType = {
     .tp_name = "DicoStrategy",
     .tp_basicsize = sizeof (PyStrategy),
     .tp_dealloc = _PyStrategy_dealloc,
-    .tp_getattr = _PyStrategy_getattr, 
+    .tp_getattr = _PyStrategy_getattr,
     .tp_repr = _PyStrategy_repr,
     .tp_str = _PyStrategy_str,
 };
@@ -312,11 +318,12 @@ _python_selector (int cmd, struct dico_key *key, const char *dict_word)
     PyObject *py_args, *py_res;
     PySelectionKey *py_key;
     void *closure = key->strat->closure;
-    
+
     py_args = PyTuple_New (3);
     PyTuple_SetItem (py_args, 0, PyLong_FromLong (cmd));
     py_key = PyObject_NEW (PySelectionKey, &PySelectionKeyType);
     py_key->key = key;
+    py_key->key_alloc = 0;
     PyTuple_SetItem (py_args, 1, (PyObject*)py_key);
     PyTuple_SetItem (py_args, 2, PyUnicode_FromString (dict_word));
 
@@ -340,7 +347,7 @@ dico_register_strat (PyObject *self, PyObject *args)
     char *fnc = NULL;
 
     if (!PyArg_ParseTuple (args, "ss|s", &name, &descr, &fnc))
-        return NULL;
+	return NULL;
 
     strat.name = name;
     strat.descr = descr;
@@ -364,7 +371,7 @@ dico_register_markup (PyObject *self, PyObject *py_obj)
 
     if (!PyUnicode_Check (py_obj)) {
 	PyErr_SetString (PyExc_TypeError, _("This parameter must be a string"));
-        return NULL;
+	return NULL;
     }
 
     type = strdup (PyUnicode_AsUTF8 (py_obj));
@@ -483,7 +490,7 @@ mod_init_db (const char *dbname, int argc, char **argv)
 {
     int pindex;
     struct _python_database *db;
-    PyObject *py_err, *py_out, *py_name, *py_module, *py_class;
+    PyObject *py_name, *py_module, *py_class;
     PyThreadState *py_ths;
 
     if (dico_parseopt (init_option, argc, argv, DICO_PARSEOPT_PERMUTE,
@@ -498,7 +505,7 @@ mod_init_db (const char *dbname, int argc, char **argv)
 
     db = malloc (sizeof (*db));
     if (!db) {
-        DICO_LOG_ERRNO();
+	DICO_LOG_ERRNO();
 	return NULL;
     }
     db->dbname = dbname;
@@ -508,7 +515,7 @@ mod_init_db (const char *dbname, int argc, char **argv)
     py_ths = Py_NewInterpreter ();
     if (!py_ths) {
 	dico_log (L_ERR, 0,
-		  _("mod_init_db: cannot create new interpreter: %s"), 
+		  _("mod_init_db: cannot create new interpreter: %s"),
 		  init_script);
 	return NULL;
     }
@@ -517,7 +524,7 @@ mod_init_db (const char *dbname, int argc, char **argv)
     db->py_ths = py_ths;
 
     dico_module_init ();
-    
+
     PyRun_SimpleString ("import sys");
     if (load_path)
 	insert_load_path (load_path);
@@ -531,7 +538,7 @@ mod_init_db (const char *dbname, int argc, char **argv)
     Py_DECREF (py_name);
 
     if (!py_module) {
-	dico_log (L_ERR, 0, _("mod_init_db: cannot load init script: %s"), 
+	dico_log (L_ERR, 0, _("mod_init_db: cannot load init script: %s"),
 		  init_script);
 	if (PyErr_Occurred ())
 	    PyErr_Print ();
@@ -629,7 +636,7 @@ _mod_get_text (PyObject *py_instance, const char *method)
 	return NULL;
     else if (!PyObject_HasAttrString (py_instance, method))
 	return NULL;
-    
+
     py_fnc = PyObject_GetAttrString (py_instance, method);
     if (py_fnc && PyCallable_Check (py_fnc)) {
 	py_res = PyObject_CallObject (py_fnc, NULL);
@@ -669,7 +676,7 @@ _tuple_to_langlist (PyObject *py_obj)
 
     if (!py_obj)
 	return NULL;
-    
+
     if (PyUnicode_Check (py_obj)) {
 	char *text = strdup (PyUnicode_AsUTF8 (py_obj));
 	list = dico_list_create ();
@@ -677,7 +684,7 @@ _tuple_to_langlist (PyObject *py_obj)
     } else if (PyTuple_Check (py_obj) || PyList_Check (py_obj)) {
 	PyObject *py_item;
 	PyObject *py_iterator = PyObject_GetIter (py_obj);
-	
+
 	list = dico_list_create ();
 
 	if (py_iterator) {
@@ -725,7 +732,7 @@ mod_lang (dico_handle_t hp, dico_list_t list[2])
 		    list[0] = _tuple_to_langlist (PyTuple_GetItem (py_res, 0));
 		    list[1] = _tuple_to_langlist (PyTuple_GetItem (py_res, 1));
 		    break;
-		    
+
 		case 1:
 		    list[0] = _tuple_to_langlist (PyTuple_GetItem (py_res, 0));
 		    break;
@@ -743,7 +750,7 @@ mod_lang (dico_handle_t hp, dico_list_t list[2])
 		    list[0] = _tuple_to_langlist (PyList_GetItem (py_res, 0));
 		    list[1] = _tuple_to_langlist (PyList_GetItem (py_res, 1));
 		    break;
-		    
+
 		case 1:
 		    list[0] = _tuple_to_langlist (PyList_GetItem (py_res, 0));
 		    break;
@@ -769,51 +776,67 @@ mod_lang (dico_handle_t hp, dico_list_t list[2])
 }
 
 struct python_result {
-    struct _python_database *db;
-    PyObject *result;
+    struct _python_database *db; /* Database */
+    PyObject *result;  /* Result obtained from match_word or define_word call*/
+    PyObject *arg;     /* Arguments passed to the match_ call.  These are
+			  preserved for the life time of the result, because
+			  the returned result object might reference any of
+			  them */
 };
 
 static dico_result_t
-_make_python_result (struct _python_database *db, PyObject *res)
+_make_python_result (struct _python_database *db, PyObject *res, PyObject *arg)
 {
     struct python_result *rp = malloc (sizeof (*rp));
     if (rp) {
 	rp->db = db;
 	rp->result = res;
+	rp->arg = arg;
     }
     return (dico_result_t)rp;
 }
 
 dico_result_t
 do_match(struct _python_database *db, const dico_strategy_t strat,
-	 struct dico_key *key)
+	 const char *word)
 {
     PyStrategy *py_strat;
     PySelectionKey *py_key;
     PyObject *py_args, *py_fnc, *py_res;
-    
+
     py_key = PyObject_NEW(PySelectionKey, &PySelectionKeyType);
     if (!py_key)
 	return NULL;
-    py_key->key = key;
-    
+
+    if ((py_key->key = malloc(sizeof(*py_key->key))) == NULL) {
+	dico_log(L_ERR, 0, _("mod_match: memory allocation failed"));
+	return NULL;
+    }
+
+    if (dico_key_init(py_key->key, strat, word)) {
+	dico_log(L_ERR, 0, _("mod_match: key initialization failed"));
+	free(py_key->key);
+	py_key->key_alloc = 0;
+	return NULL;
+    }
+    py_key->key_alloc = 1;
+
     py_strat = PyObject_NEW(PyStrategy, &PyStrategyType);
     if (py_strat) {
 	py_strat->strat = strat;
-	
+
 	py_args = PyTuple_New(2);
 	PyTuple_SetItem(py_args, 0, (PyObject *)py_strat);
 	PyTuple_SetItem(py_args, 1, (PyObject *)py_key);
 	py_fnc = PyObject_GetAttrString(db->py_instance, "match_word");
 	if (py_fnc && PyCallable_Check(py_fnc)) {
 	    py_res = PyObject_CallObject(py_fnc, py_args);
-	    Py_DECREF(py_args);
 	    Py_DECREF(py_fnc);
 	    if (py_res) {
 		if (PyBool_Check(py_res) && py_res == Py_False)
-		    return NULL;
+		    /* Do nothing; NULL will be returned */;
 		else
-		    return _make_python_result(db, py_res);
+		    return _make_python_result(db, py_res, py_args);
 	    } else if (PyErr_Occurred())
 		PyErr_Print();
 	}
@@ -825,18 +848,8 @@ static dico_result_t
 mod_match (dico_handle_t hp, const dico_strategy_t strat, const char *word)
 {
     struct _python_database *db = (struct _python_database *)hp;
-    struct dico_key key;
-    dico_result_t res;
-    
     PyThreadState_Swap(db->py_ths);
-
-    if (dico_key_init(&key, strat, word)) {
-	dico_log(L_ERR, 0, _("mod_match: key initialization failed"));
-	return NULL;
-    }
-    res = do_match(db, strat, &key);
-    dico_key_deinit(&key);
-    return res;
+    return do_match(db, strat, word);
 }
 
 static dico_result_t
@@ -853,23 +866,23 @@ mod_define (dico_handle_t hp, const char *word)
     py_fnc = PyObject_GetAttrString (db->py_instance, "define_word");
     if (py_fnc && PyCallable_Check (py_fnc)) {
 	py_res = PyObject_CallObject (py_fnc, py_args);
-	Py_DECREF (py_args);
 	Py_DECREF (py_fnc);
 	if (py_res) {
 	    if (PyBool_Check (py_res) && py_res == Py_False)
-		return NULL;
+		/* Do nothing and return NULL */;
 	    else
-		return _make_python_result (db, py_res);
+		return _make_python_result (db, py_res, py_args);
 	} else if (PyErr_Occurred ())
 	    PyErr_Print ();
     }
+    Py_DECREF (py_args);
     return NULL;
 }
 
 static int
 mod_output_result (dico_result_t rp, size_t n, dico_stream_t str)
 {
-    PyObject *py_args, *py_fnc, *py_out;
+    PyObject *py_args, *py_fnc;
     struct python_result *gres = (struct python_result *)rp;
     struct _python_database *db = (struct _python_database *)gres->db;
 
@@ -889,13 +902,13 @@ mod_output_result (dico_result_t rp, size_t n, dico_stream_t str)
 
     py_fnc = PyObject_GetAttrString (db->py_instance, "output");
     if (py_fnc && PyCallable_Check (py_fnc)) {
-	/* FIXME: should we checj/propagate the retval? */
+	/* FIXME: should we check/propagate the retval? */
 	PyObject_CallObject (py_fnc, py_args);
-	Py_DECREF (py_args);
 	Py_DECREF (py_fnc);
 	if (PyErr_Occurred ())
 	    PyErr_Print ();
     }
+    Py_DECREF (py_args);
 
     stdout_info_init ();
 
@@ -980,12 +993,13 @@ mod_free_result (dico_result_t rp)
     py_fnc = PyObject_GetAttrString (db->py_instance, "free_result");
     if (py_fnc && PyCallable_Check (py_fnc)) {
 	PyObject_CallObject (py_fnc, py_args);
-	Py_DECREF (py_args);
 	Py_DECREF (py_fnc);
 	if (PyErr_Occurred ())
 	    PyErr_Print ();
     }
+    Py_DECREF (py_args);
     Py_DECREF (gres->result);
+    Py_DECREF (gres->arg);
     free (gres);
 }
 
@@ -1040,23 +1054,23 @@ mod_result_headers (dico_result_t rp, dico_assoc_list_t hdr)
 
     py_args = PyTuple_New (2);
     PyTuple_SetItem (py_args, 0, gres->result);
-    PyTuple_SetItem (py_args, 1, py_dict);
     Py_INCREF (gres->result);
+    PyTuple_SetItem (py_args, 1, py_dict);
 
     py_fnc = PyObject_GetAttrString (db->py_instance, "result_headers");
     if (py_fnc && PyCallable_Check (py_fnc)) {
 	py_res = PyObject_CallObject (py_fnc, py_args);
-	Py_DECREF (py_args);
 	Py_DECREF (py_fnc);
 	if (py_res && PyDict_Check (py_res)) {
 	    _dict_to_assoc (hdr, py_res);
+
 	    Py_DECREF (py_res);
 	} else if (PyErr_Occurred ()) {
 	    PyErr_Print ();
 	    return 1;
 	}
     }
-    Py_DECREF (py_dict);
+    Py_DECREF (py_args);
     return 0;
 }
 
